==> zgo-v1.12.0.zip: zgo-v1.12.0/zhttp/zhttp.go <==
package zhttp

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"time"

	"git.barsukov.pro/barsukov/zgo/zjson"
	"git.barsukov.pro/barsukov/zgo/zutils"
)

var DefaultClient = &http.Client{Timeout: time.Second * 10}

type ZClient struct {
	httpClient *http.Client
}

type Config struct {
	Repit int
}

func New(c *http.Client) *ZClient {
	return &ZClient{httpClient: c}
}

func Default() *ZClient {
	return New(DefaultClient)
}

func (c *ZClient) Do(r *http.Request, try int, sleepDur time.Duration) (resp *http.Response, err error) {
	for i := 0; i < try; i++ {
		if resp, err = c.httpClient.Do(r); err != nil {
			if i+1 < try {
				time.Sleep(sleepDur)
			}

			continue
		}

		break
	}

	return
}

func (c *ZClient) GetEmpty(url string) error {
	var err error
	var resp *http.Response

	if resp, err = c.httpClient.Get(url); err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode >= 200 && resp.StatusCode < 300 {
		return nil
	}

	return fmt.Errorf("status code: %v", resp.StatusCode)
}
func (c *ZClient) GetJson(target any, url string) error {
	var err error
	var resp *http.Response

	if resp, err = c.httpClient.Get(url); err != nil {
		return err
	}
	defer resp.Body.Close()

	return json.NewDecoder(resp.Body).Decode(&target)
}
func (c *ZClient) PostJson(target any, url string, body *bytes.Reader, headers map[string]any) error {
	r, _ := http.NewRequest("POST", url, body)

	if headers != nil {
		if _, ok := headers["Content-Type"]; !ok {
			headers["Content-Type"] = "application/json"
		}
	}

	for k, v := range headers {
		r.Header.Set(k, fmt.Sprintf("%v", v))
	}
	resp, err := c.httpClient.Do(r)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if !zutils.InArray(resp.StatusCode, []int{http.StatusOK, http.StatusAccepted, http.StatusAccepted}) {
		b, _ := io.ReadAll(resp.Body)
		return fmt.Errorf("%s", b)
	}

	return json.NewDecoder(resp.Body).Decode(target)
}

func GetJson(target any, url string) error {
	return Default().GetJson(target, url)
}
func GetEmpty(url string) error {
	return Default().GetEmpty(url)
}
func PostJson(target any, url string, body *bytes.Reader, headers map[string]any) error {
	return Default().PostJson(target, url, body, headers)
}
func PostJsonObj(target any, url string, body any, headers map[string]any) error {
	var bodyReader *bytes.Reader
	switch t := body.(type) {
	case string:
		bodyReader = bytes.NewReader([]byte(t))
	default:
		bodyReader = zjson.BytesReader(body)
	}

	return PostJson(target, url, bodyReader, headers)
}

==> zgo-v1.12.1.zip: zgo-v1.12.1/zhttp/zhttp.go <==
package zhttp

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"time"

	"git.barsukov.pro/barsukov/zgo/zjson"
	"git.barsukov.pro/barsukov/zgo/zutils"
)

var DefaultClient = &http.Client{Timeout: time.Second * 10}

type ZClient struct {
	httpClient *http.Client
}

type Config struct {
	Repit int
}

func New(c *http.Client) *ZClient {
	return &ZClient{httpClient: c}
}

func Default() *ZClient {
	return New(DefaultClient)
}

func (c *ZClient) Do(r *http.Request, try int, sleepDur time.Duration) (resp *http.Response, err error) {
	for i := 0; i < try; i++ {
		if resp, err = c.httpClient.Do(r); err != nil {
			if i+1 < try {
				time.Sleep(sleepDur)
			}

			continue
		}

		break
	}

	return
}

func (c *ZClient) GetEmpty(url string) error {
	var err error
	var resp *http.Response

	if resp, err = c.httpClient.Get(url); err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode >= 200 && resp.StatusCode < 300 {
		return nil
	}

	return fmt.Errorf("status code: %v", resp.StatusCode)
}
func (c *ZClient) GetJson(target any, url string) error {
	var err error
	var resp *http.Response

	if resp, err = c.httpClient.Get(url); err != nil {
		return err
	}
	defer resp.Body.Close()

	return json.NewDecoder(resp.Body).Decode(&target)
}
func (c *ZClient) PostJson(target any, url string, body *bytes.Reader, headers map[string]any) error {
	r, _ := http.NewRequest("POST", url, body)

	if headers != nil {
		if _, ok := headers["Content-Type"]; !ok {
			headers["Content-Type"] = "application/json"
		}
	}

	for k, v := range headers {
		r.Header.Set(k, fmt.Sprintf("%v", v))
	}
	resp, err := c.httpClient.Do(r)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if !zutils.InArray(resp.StatusCode, []int{http.StatusOK, http.StatusAccepted, http.StatusAccepted}) {
		b, _ := io.ReadAll(resp.Body)
		return fmt.Errorf("%s", b)
	}

	return json.NewDecoder(resp.Body).Decode(target)
}

func GetJson(target any, url string) error {
	return Default().GetJson(target, url)
}
func GetEmpty(url string) error {
	return Default().GetEmpty(url)
}
func PostJson(target any, url string, body *bytes.Reader, headers map[string]any) error {
	return Default().PostJson(target, url, body, headers)
}
func PostJsonObj(target any, url string, body any, headers map[string]any) error {
	var bodyReader *bytes.Reader
	switch t := body.(type) {
	case string:
		bodyReader = bytes.NewReader([]byte(t))
	default:
		bodyReader = zjson.BytesReader(body)
	}

	return PostJson(target, url, bodyReader, headers)
}

func Ok(r *http.Response) bool {
	return OkCode(r.StatusCode)
}
func OkCode(code int) bool {
	return code >= 200 && code < 300
}
